# coding: utf-8

require 'fluent/plugin/output'
require 'nsq'
require 'yajl'

module Fluent::Plugin
  class NSQOutput < Output
    Fluent::Plugin.register_output('nsq', self)

    config_param :topic, :string, default: nil
    config_param :nsqlookupd, :array, default: nil
    config_param :nsqd, :array, default: nil
    config_param :enable_tls, :bool, default: false
    config_param :key, :string, default: nil
    config_param :certificate, :string, default: nil
    config_param :ca_certificate, :string, default: nil

    config_section :buffer do
      config_set_default :chunk_keys, ['tag']
    end

    def configure(conf)
      super

      fail Fluent::ConfigError, 'Missing nsqlookupd or nsqd' unless @nsqlookupd || @nsqd
      fail Fluent::ConfigError, 'Missing topic' unless @topic
      if @enable_tls
        fail Fluent::ConfigError, 'Missing tls config params' unless @key && @certificate && @ca_certificate
      end
    end

    def start
      super
      producer_config = {topic: @topic}

      if  @nsqlookupd
        producer_config[:nsqlookupd] = @nsqlookupd
      elsif @nsqd
        producer_config[:nsqd] = @nsqd
      end

      if @enable_tls
        producer_config[:tls_v1] = true
        producer_config[:tls_options] = {
              key: @key,
              certificate: @certificate,
              ca_certificate: @ca_certificate,
              verify_mode: OpenSSL::SSL::VERIFY_PEER
          }
      end
      
      @producer = Nsq::Producer.new(producer_config)
    end

    def shutdown
      @producer.terminate
      super
    end

    def write(chunk)
      return if chunk.empty?

      tag = chunk.metadata.tag
      chunk.each do |time, record|
        tagged_record = record.merge(
          :_key => tag,
          :_ts => time.to_f,
          :'@timestamp' => Time.at(time).iso8601(3) # kibana/elasticsearch friendly
        )
        begin
          @producer.write(Yajl.dump(tagged_record))
        rescue => e
          log.warn("nsq: #{e}")
        end
      end
    end
  end
end
